/* (C) 2008 The Measurement Factory */

#ifndef LIBECAP__COMMON__AREA_H
#define LIBECAP__COMMON__AREA_H

#include <libecap/common/forward.h>
#include <libecap/common/memory.h>
#include <string>

namespace libecap {

typedef std::string::size_type size_type;
extern const size_type nsize; // same as std::string::npos

class AreaDetails;

// a continiuos, fixed-size buffer area
// no zero-termination is guaranteed
class Area {
	public:
		typedef shared_ptr<AreaDetails> Details;

	public:
		static Area FromTempBuffer(const char *aStart, size_type aSize);
		static Area FromTempString(const std::string &tmp);

		Area(): start(""), size(0) {}
		Area(const char *aStart, size_type aSize):
			start(aStart), size(aSize) {}
		Area(const char *aStart, size_type aSize, const Details &aDetails):
			start(aStart), size(aSize), details(aDetails) {}

		std::string toString() const; // expensive

	public:
		const char *start;
		size_type size;

		Details details; // creator-defined
};

// this stub can be enhanced by area creators to optimize area operations
class AreaDetails {
	public:
		virtual ~AreaDetails() {}
};

} // namespace libecap

#endif
